@@ -1,8 +1,5 @@
 BiCDF <- function (u1, u2, family, par1, par2 = NULL, test = TRUE){
 
-  epsilon <- 0.0000001 # 0.9999999 0.0001 # sqrt(.Machine$double.eps)
-  max.p   <- 0.9999999
-
 if(test == TRUE){
 
     if(is.null(u1) == TRUE || is.null(u2) == TRUE) stop("The margins are not set or have length zero.")
@@ -31,9 +28,7 @@ if(test == TRUE){
     if(family %in% c(3,7,11))           res <- u2 - BCDF(1 - u1, u2, family, -par1)                  # 90
     if(family %in% c(5,9,13))           res <- u1 - BCDF(u1, 1 - u2, family, -par1)                  # 270
 
-    res <- ifelse(res > max.p, max.p, res) 
-    res <- ifelse(res < epsilon, epsilon, res) 
-
+    res <- mm(res)
     res
       
 }
