@@ -1,4 +1,4 @@
-bprobgHsContUniv3 <- function(params, respvec, VC, sp = NULL, qu.mag = NULL, AT = FALSE){
+bprobgHsContUniv3 <- function(params, respvec, VC, ps, AT = FALSE){
 
 epsilon <- 0.0000001 # 0.9999999 0.0001 # sqrt(.Machine$double.eps)
 
@@ -26,11 +26,13 @@ sstr1 <- esp.tr(sigma2.st, VC$margins[2])
 sigma2.st <- sstr1$vrb.st 
 sigma2    <- sstr1$vrb 
     
-sstr1 <- esp.tr(nu.st, VC$margins[2])  
+sstr1 <- enu.tr(nu.st, VC$margins[2])  
 nu.st <- sstr1$vrb.st 
 nu    <- sstr1$vrb 
 
-dHs  <- distrHs(y2, eta2, sigma2, sigma2.st, nu, nu.st, margin2=VC$margins[2], naive = TRUE)
+if(VC$margins[2] %in% VC$m3)  dHs  <-      distrHs(y2, eta2, sigma2, sigma2.st, nu, nu.st, margin2=VC$margins[2], naive = TRUE)
+if(VC$margins[2] %in% VC$m3d) dHs  <- distrHsDiscr(y2, eta2, sigma2, sigma2.st, nu, nu.st, margin2=VC$margins[2], naive = TRUE, y2m = VC$y2m)
+
 
 }
 
@@ -52,11 +54,12 @@ sstr1 <- esp.tr(sigma2.st, VC$margins[1])
 sigma2.st <- sstr1$vrb.st 
 sigma2    <- sstr1$vrb 
     
-sstr1 <- esp.tr(nu.st, VC$margins[1])  
+sstr1 <- enu.tr(nu.st, VC$margins[1])  
 nu.st <- sstr1$vrb.st 
 nu    <- sstr1$vrb           
  
-dHs  <- distrHs(respvec$y1, eta2, sigma2, sigma2.st, nu, nu.st, margin2=VC$margins[1], naive = TRUE)
+if(VC$margins[1] %in% VC$m3)  dHs <-      distrHs(respvec$y1, eta2, sigma2, sigma2.st, nu, nu.st, margin2=VC$margins[1], naive = TRUE)
+if(VC$margins[1] %in% VC$m3d) dHs <- distrHsDiscr(respvec$y1, eta2, sigma2, sigma2.st, nu, nu.st, margin2=VC$margins[2], naive = TRUE, y2m = VC$y1m)
 
 }
 
@@ -68,27 +71,34 @@ eta2 <- eta.tr(eta2, VC$margins[2])
     
 
 if(is.null(VC$X3))  sigma2.st <- params[(VC$X2.d2 + 1)] 
-if(!is.null(VC$X3)) sigma2.st <- VC$X4%*%params[(VC$X2.d2+1):(VC$X2.d2+VC$X4.d2)]
+if(is.null(VC$X4))  nu.st     <- params[(VC$X2.d2 + 2)] 
 
-if(is.null(VC$X4))  nu.st <- params[(VC$X2.d2 + 2)] 
 
+if(!is.null(VC$X3)){
+
+if( VC$margins[1] %in% c(VC$m2,VC$m2d,VC$m3) ) sigma2.st <- VC$X4%*%params[(VC$X2.d2+1):(VC$X2.d2+VC$X4.d2)]
+if( VC$margins[1] %in% c(VC$m1d) )             sigma2.st <- VC$X3%*%params[(VC$X2.d2+1):(VC$X2.d2+VC$X3.d2)]
 
 if(!is.null(VC$X4)){
-
-if( VC$margins[1] %in% c("DAGUM","SM")  ) nu.st <- VC$X6%*%params[(VC$X2.d2+VC$X4.d2+1):(VC$X2.d2+VC$X4.d2+VC$X6.d2)] 
-if(!(VC$margins[1] %in% c("DAGUM","SM"))) nu.st <- VC$X5%*%params[(VC$X2.d2+VC$X4.d2+1):(VC$X2.d2+VC$X4.d2+VC$X5.d2)] 
-
+if( VC$margins[1] %in% c(VC$m3)        ) nu.st <- VC$X6%*%params[(VC$X2.d2+VC$X4.d2+1):(VC$X2.d2+VC$X4.d2+VC$X6.d2)] 
+if( VC$margins[1] %in% c(VC$m2,VC$m2d) ) nu.st <- VC$X5%*%params[(VC$X2.d2+VC$X4.d2+1):(VC$X2.d2+VC$X4.d2+VC$X5.d2)] 
+if( VC$margins[1] %in% c(VC$m1d) )       nu.st <- VC$X4%*%params[(VC$X2.d2+VC$X3.d2+1):(VC$X2.d2+VC$X3.d2+VC$X5.d2)] 
+                   }
 }
 
+
+
+
 sstr1 <- esp.tr(sigma2.st, VC$margins[2])  
 sigma2.st <- sstr1$vrb.st 
 sigma2    <- sstr1$vrb 
     
-sstr1 <- esp.tr(nu.st, VC$margins[2])  
+sstr1 <- enu.tr(nu.st, VC$margins[2])  
 nu.st <- sstr1$vrb.st 
 nu    <- sstr1$vrb          
  
-dHs  <- distrHs(respvec$y2, eta2, sigma2, sigma2.st, nu, nu.st, margin2=VC$margins[2], naive = TRUE)
+if(VC$margins[2] %in% VC$m3)  dHs <-      distrHs(respvec$y2, eta2, sigma2, sigma2.st, nu, nu.st, margin2=VC$margins[2], naive = TRUE)
+if(VC$margins[2] %in% VC$m3d) dHs <- distrHsDiscr(respvec$y2, eta2, sigma2, sigma2.st, nu, nu.st, margin2=VC$margins[2], naive = TRUE, y2m = VC$y2m)
 
 }
 
@@ -171,7 +181,6 @@ if( !is.null(VC$X3) ){
               cbind( t(be.nu),    t(sa.nu),  si.nu ) )       }
 
 
-if( (VC$l.sp2==0 && VC$l.sp3==0 && VC$l.sp4==0) || VC$fp==TRUE) ps <- list(S.h = 0, S.h1 = 0, S.h2 = 0) else ps <- pen(params, qu.mag, sp, VC, respvec$univ)
 
 }
 
@@ -215,7 +224,6 @@ if( !is.null(VC$X3) ){
 
 
 
-if( (VC$l.sp1==0 && VC$l.sp3==0 && VC$l.sp5==0) || VC$fp==TRUE) ps <- list(S.h = 0, S.h1 = 0, S.h2 = 0) else ps <- pen(params, qu.mag, sp, VC, respvec$univ)
 
 }
 
@@ -239,18 +247,15 @@ if( is.null(VC$X3) ){
 
   H <- rbind( cbind( be.be      , be.sigma,             be.nu             ), 
               cbind( t(be.sigma), sum(d2l.sigma.sigma), sum(d2l.sigma.nu) ),
-              cbind( t(be.nu),    sum(d2l.sigma.nu),    sum(d2l.nu.nu)    )  )  
-              
-              
-}
+              cbind( t(be.nu),    sum(d2l.sigma.nu),    sum(d2l.nu.nu)    )  )    }
 
 
 
-if( !is.null(VC$X3) ){
+if( !is.null(VC$X3) ){###
 
 
 
-if( VC$margins[1] %in% c("DAGUM","SM")  ){
+if( VC$margins[1] %in% VC$m3  ){
 
   G   <- c( colSums( c(dl.dbe)*VC$X2      ),
             colSums( c(dl.dsigma.st)*VC$X4),
@@ -271,7 +276,7 @@ if( VC$margins[1] %in% c("DAGUM","SM")  ){
 
 
 
-if( !(VC$margins[1] %in% c("DAGUM","SM"))  ){
+if( VC$margins[1] %in% c(VC$m2,VC$m2d)  ){
 
   G   <- c( colSums( c(dl.dbe)*VC$X2      ),
             colSums( c(dl.dsigma.st)*VC$X4),
@@ -291,11 +296,32 @@ if( !(VC$margins[1] %in% c("DAGUM","SM"))  ){
               cbind( t(be.nu),    t(sa.nu),  si.nu ) )   }
 
 
-}
 
 
-if( VC$margins[1] %in% c("DAGUM","SM")  ) { if( (VC$l.sp2==0 && VC$l.sp4==0 && VC$l.sp6==0) || VC$fp==TRUE) ps <- list(S.h = 0, S.h1 = 0, S.h2 = 0) else ps <- pen(params, qu.mag, sp, VC, respvec$univ) }
-if( !(VC$margins[1] %in% c("DAGUM","SM"))  ) { if( (VC$l.sp2==0 && VC$l.sp4==0 && VC$l.sp5==0) || VC$fp==TRUE) ps <- list(S.h = 0, S.h1 = 0, S.h2 = 0) else ps <- pen(params, qu.mag, sp, VC, respvec$univ) }
+if( VC$margins[1] %in% c(VC$m1d)  ){
+
+  G   <- c( colSums( c(dl.dbe)*VC$X2      ),
+            colSums( c(dl.dsigma.st)*VC$X3),
+            colSums( c(dl.dnu.st)*VC$X4   )  )
+                
+  be.be    <- crossprod(VC$X2*c(d2l.be.be),VC$X2)
+  be.sigma <- crossprod(VC$X2*c(d2l.be.sigma),VC$X3)
+  be.nu    <- crossprod(VC$X2*c(d2l.be.nu),VC$X4)
+  
+  si.sigma <- crossprod(VC$X3*c(d2l.sigma.sigma),VC$X3)  
+  si.nu    <- crossprod(VC$X4*c(d2l.nu.nu),VC$X4)    
+  
+  sa.nu    <- crossprod(VC$X3*c(d2l.sigma.nu),VC$X4)    
+  
+  H <- rbind( cbind( be.be      , be.sigma , be.nu ), 
+              cbind( t(be.sigma), si.sigma,  sa.nu ),
+              cbind( t(be.nu),    t(sa.nu),  si.nu ) ) }
+
+
+} ###
+
+
+
 
 
 }
@@ -304,15 +330,25 @@ if( !(VC$margins[1] %in% c("DAGUM","SM"))  ) { if( (VC$l.sp2==0 && VC$l.sp4==0 &
 
 if(VC$extra.regI == "pC") H <- regH(H, type = 1)
   
-         S.res <- res
-         res   <- S.res + ps$S.h1
-         G     <- G + ps$S.h2
-         H     <- H + ps$S.h  
+  S.h  <- ps$S.h  
+
+
+  if( length(S.h) != 1){
+  
+  S.h1 <- 0.5*crossprod(params,S.h)%*%params
+  S.h2 <- S.h%*%params
+  
+  } else S.h <- S.h1 <- S.h2 <- 0   
+  
+  S.res <- res
+  res   <- S.res + S.h1
+  G     <- G + S.h2
+  H     <- H + S.h   
         
 if(VC$extra.regI == "sED") H <- regH(H, type = 2) 
   
 
-list(value=res, gradient=G, hessian=H, S.h=ps$S.h, l=S.res, l.par=l.par, 
+list(value=res, gradient=G, hessian=H, S.h=S.h, S.h1=S.h1, S.h2=S.h2, l=S.res, l.par=l.par, 
      ps = ps, sigma2.st = sigma2.st, nu.st = nu.st,
      BivD=VC$BivD, eta2 = eta2, sigma2 = sigma2, nu = nu)      
 
