@@ -1,4 +1,4 @@
-bprobgHsSS <- function(params, respvec, VC, sp = NULL, qu.mag = NULL){
+bprobgHsSS <- function(params, respvec, VC, ps){
 
   epsilon <- 0.0000001
   max.p   <- 0.9999999
@@ -203,15 +203,23 @@ bit2.b1b1a     <- -c.copula2.be1a*d.n1^2  + (1-c.copula.be1a)*der2p1.dereta12
     
  res <- -sum(l.par)   
     
-if( ( VC$l.sp1==0 && VC$l.sp2==0 && VC$l.sp3==0) || VC$fp==TRUE) ps <- list(S.h = 0, S.h1 = 0, S.h2 = 0) else ps <- pen(params, qu.mag, sp, VC)
 
 
 if(VC$extra.regI == "pC" && VC$hess==FALSE) H <- regH(H, type = 1)
   
-         S.res <- res
-         res   <- S.res + ps$S.h1
-         G     <- G + ps$S.h2
-         H     <- H + ps$S.h  
+  S.h  <- ps$S.h  
+  
+  if( length(S.h) != 1){
+  
+  S.h1 <- 0.5*crossprod(params,S.h)%*%params
+  S.h2 <- S.h%*%params
+  
+  } else S.h <- S.h1 <- S.h2 <- 0   
+  
+  S.res <- res
+  res   <- S.res + S.h1
+  G     <- G + S.h2
+  H     <- H + S.h  
         
 if(VC$extra.regI == "sED") H <- regH(H, type = 2)   
   
@@ -229,7 +237,7 @@ dl.dbe2t[VC$inde]  <- dl.dbe2	;dl.dbe2  <- dl.dbe2t
 dl.drhot[VC$inde]  <- dl.drho	;dl.drho  <- dl.drhot
 
 
-         list(value=res, gradient=G, hessian=H, S.h=ps$S.h, l=S.res, l.par=l.par, ps = ps,
+         list(value=res, gradient=G, hessian=H, S.h=S.h, S.h1=S.h1, S.h2=S.h2, l=S.res, l.par=l.par, ps = ps,
               p11=p11, p10=p10, p0=p0, eta1=eta1, eta2=eta2, etad=etad,
               dl.dbe1=dl.dbe1, dl.dbe2=dl.dbe2, dl.drho=dl.drho,
               BivD=VC$BivD, p1=p1)     
@@ -239,27 +247,4 @@ dl.drhot[VC$inde]  <- dl.drho	;dl.drho  <- dl.drhot
 
 
 
-     
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
+   
\ No newline at end of file
