@@ -10,7 +10,7 @@ if (min(e.v) > 0) cat("\n",mv," information matrix is positive definite\n",sep="
 
 cat("Eigenvalue range: [",min(e.v),",",max(e.v),"]\n", sep = "")
 
-if( (x$l.sp1!=0 || x$l.sp2!=0 || x$l.sp3!=0 || x$l.sp4!=0 || x$l.sp5!=0 || x$l.sp6!=0) && x$fp==FALSE ){
+if( (x$l.sp1!=0 || x$l.sp2!=0 || x$l.sp3!=0 || x$l.sp4!=0 || x$l.sp5!=0 || x$l.sp6!=0 || x$l.sp7!=0) && x$fp==FALSE ){
 
 cat("\nTrust region iterations before smoothing parameter estimation:",x$iter.if)
 cat("\nLoops for smoothing parameter estimation:",x$iter.sp) 
@@ -19,7 +19,7 @@ cat("\nTrust region iterations within smoothing loops:",x$iter.inner,"\n\n")
 }else{cat("\nTrust region iterations:",x$iter.if,"\n\n")}
 
 if(!is.null(x$conv.sp)){
-if(x$conv.sp == FALSE) cat("Smoothing algorithm reached the max. number of iterations allowed.\n\n")
+if(x$iter.sp >= x$iterlimsp) cat("Smoothing algorithm reached the max. number of loops allowed.\n\n")
 }
 
 
