@@ -6,8 +6,10 @@ mub <- c("BE")
 
 if( !(margin %in% mub) ){
  
-   vrb.st <- ifelse( vrb.st > 28,   28, vrb.st )  
-   vrb.st <- ifelse( vrb.st < -17, -17, vrb.st ) 
+   vrb.st <- ifelse( vrb.st > 20,   20, vrb.st )  # it was 28
+   #vrb.st <- ifelse( vrb.st < -6.90, -6.90, vrb.st ) # -14.5, 6.90
+   vrb.st <- ifelse( vrb.st < -14.5, -14.5, vrb.st ) 
+   
    vrb    <- exp(vrb.st)
     
 }
