@@ -1,11 +1,11 @@
 eta.tr <- function(vrb.st, margin){
 
-mupos <- c("LN","WEI","iG","GA","DAGUM","SM","FISK")
+mupos <- c("LN","WEI","iG","GA","DAGUM","SM","FISK","NBI","NBII","PO","ZTP","PIG")
 mub   <- c("BE")
 
 if(margin %in% mupos){
-   vrb.st <- ifelse( vrb.st > 28,   28, vrb.st )  
-   vrb.st <- ifelse( vrb.st < -17, -17, vrb.st ) 
+   vrb.st <- ifelse( vrb.st > 20,   20, vrb.st )  # it was 28
+   vrb.st <- ifelse( vrb.st < -9, -9, vrb.st ) # it was -17
 }
 
 if(margin %in% mub){
@@ -16,4 +16,7 @@ if(margin %in% mub){
 
 vrb.st  
  
-}    
\ No newline at end of file
+} 
+
+
+
