@@ -1,8 +1,9 @@
 print.summary.copulaReg <- function(x, digits = max(3, getOption("digits") - 3),
                                              signif.stars = getOption("show.signif.stars"),...){
 
-   cont2par <- c("N","GU","rGU","LO","LN","WEI","iG","GA","GAi","BE","FISK")  
-   cont3par <- c("DAGUM","SM") 
+   cont1par <- c("PO","ZTP")  
+   cont2par <- c("N","GU","rGU","LO","LN","WEI","iG","GA","GAi","BE","FISK","NBI","NBII","NBIa","NBIIa","PIG")  
+   cont3par <- c("DAGUM","SM","DEL","SICHEL") 
  
   if(x$BivD=="FGM")  {cop <- "FGM"                ;lind <- "atanh"} 
   if(x$BivD=="AMH")  {cop <- "AMH"                ;lind <- "atanh"} 
@@ -33,10 +34,10 @@ print.summary.copulaReg <- function(x, digits = max(3, getOption("digits") - 3),
  
    if(x$margins[1] %in% c("N","GU","rGU","LO","GAi") )  m1l <- "identity"
    if(x$margins[2] %in% c("N","GU","rGU","LO","GAi") )  m2l <- "identity"
-   if(x$margins[1] %in% c("LN","WEI","iG","GA","DAGUM","SM","FISK") ) m1l <- "log" 
-   if(x$margins[2] %in% c("LN","WEI","iG","GA","DAGUM","SM","FISK") ) m2l <- "log" 
-   if(x$margins[1] %in% c("BE") )                              m1l <- "qlogis" 
-   if(x$margins[2] %in% c("BE") )                              m2l <- "qlogis"    
+   if(x$margins[1] %in% c("LN","WEI","iG","GA","DAGUM","SM","FISK","NBI","NBII","NBIa","NBIIa","PIG","PO","ZTP") ) m1l <- "log" 
+   if(x$margins[2] %in% c("LN","WEI","iG","GA","DAGUM","SM","FISK","NBI","NBII","NBIa","NBIIa","PIG","PO","ZTP") ) m2l <- "log" 
+   if(x$margins[1] %in% c("BE") )                       m1l <- "qlogis" 
+   if(x$margins[2] %in% c("BE") )                       m2l <- "qlogis"    
    
 
   
@@ -49,14 +50,18 @@ print.summary.copulaReg <- function(x, digits = max(3, getOption("digits") - 3),
   if(x$margins[1]=="LN")     cat("\nMARGIN 1: log-normal") 
   if(x$margins[1]=="WEI")    cat("\nMARGIN 1: Weibull") 
   if(x$margins[1]=="iG")     cat("\nMARGIN 1: inverse Gaussian") 
-  if(x$margins[1]%in%c("GA","GAi"))     cat("\nMARGIN 1: gamma")  
+  if(x$margins[1]%in%c("GA","GAi"))  cat("\nMARGIN 1: gamma")  
   if(x$margins[1]=="BE")     cat("\nMARGIN 1: beta")    
   if(x$margins[1]=="DAGUM")  cat("\nMARGIN 1: Dagum")  
   if(x$margins[1]=="SM")     cat("\nMARGIN 1: Singh-Maddala") 
-  if(x$margins[1]=="FISK")     cat("\nMARGIN 1: Fisk") 
+  if(x$margins[1]=="FISK")   cat("\nMARGIN 1: Fisk") 
+  if(x$margins[1]%in%c("NBI","NBIa"))    cat("\nMARGIN 1: Negative Binomial - Type I") 
+  if(x$margins[1]%in%c("NBII","NBIIa"))   cat("\nMARGIN 1: Negative Binomial - Type II")  
+  if(x$margins[1]=="PIG")    cat("\nMARGIN 1: Poisson inverse Gaussian")
+  if(x$margins[1]=="PO")     cat("\nMARGIN 1: Poisson")   
+  if(x$margins[1]=="ZTP")    cat("\nMARGIN 1: Zero Truncated Poisson")    
   
   
-
   if(x$margins[2]=="N")      cat("\nMARGIN 2: Gaussian")  
   if(x$margins[2]=="GU")     cat("\nMARGIN 2: Gumbel")    
   if(x$margins[2]=="rGU")    cat("\nMARGIN 2: reverse Gumbel")  
@@ -64,12 +69,17 @@ print.summary.copulaReg <- function(x, digits = max(3, getOption("digits") - 3),
   if(x$margins[2]=="LN")     cat("\nMARGIN 2: log-normal") 
   if(x$margins[2]=="WEI")    cat("\nMARGIN 2: Weibull") 
   if(x$margins[2]=="iG")     cat("\nMARGIN 2: inverse Gaussian") 
-  if(x$margins[2]%in%c("GA","GAi"))     cat("\nMARGIN 2: gamma")   
+  if(x$margins[2]%in%c("GA","GAi")) cat("\nMARGIN 2: gamma")   
   if(x$margins[2]=="BE")     cat("\nMARGIN 2: beta")    
   if(x$margins[2]=="DAGUM")  cat("\nMARGIN 2: Dagum")
   if(x$margins[2]=="SM")     cat("\nMARGIN 2: Singh-Maddala") 
   if(x$margins[2]=="FISK")   cat("\nMARGIN 2: Fisk") 
-  
+  if(x$margins[2]%in%c("NBI","NBIa"))    cat("\nMARGIN 2: Negative Binomial - Type I") 
+  if(x$margins[2]%in%c("NBII","NBIIa"))   cat("\nMARGIN 2: Negative Binomial - Type II")  
+  if(x$margins[2]=="PIG")    cat("\nMARGIN 2: Poisson inverse Gaussian")  
+  if(x$margins[2]=="PO")     cat("\nMARGIN 2: Poisson")   
+  if(x$margins[2]=="ZTP")    cat("\nMARGIN 2: Zero Truncated Poisson")    
+
   
   
   
@@ -162,6 +172,133 @@ print.summary.copulaReg <- function(x, digits = max(3, getOption("digits") - 3),
 
 
 }
+
+
+
+
+
+
+
+
+     if( x$margins[1] %in% cont1par && x$margins[2] %in% cont2par ){
+
+ 
+  cat("\nEQUATION 3")
+  if(x$margins[2] !="BE") cat("\nLink function for sigma2.2:","log","\n") else cat("\nLink function for sigma2.2:","qlogis","\n")  
+  cat("Formula: "); print(x$formula[[3]]) 
+    cat("\n")
+      cat("Parametric coefficients:\n")
+      printCoefmat(x$tableP3,digits = digits, signif.stars = signif.stars,na.print = "NA",...)
+      cat("\n")
+    
+        if(x$l.sp3!=0){
+        cat("Smooth components' approximate significance:\n")
+        printCoefmat(x$tableNP3,digits = digits, signif.stars = signif.stars,has.Pvalue = TRUE,na.print = "NA",cs.ind = 1,...)
+        cat("\n")
+      }
+  
+  cat("\nEQUATION 4")
+  cat("\nLink function for theta:",lind,"\n") 
+  cat("Formula: "); print(x$formula[[4]])
+    cat("\n")
+      cat("Parametric coefficients:\n")
+      printCoefmat(x$tableP4,digits = digits, signif.stars = signif.stars,na.print = "NA",...)
+      cat("\n")
+    
+        if(x$l.sp4!=0){
+        cat("Smooth components' approximate significance:\n")
+        printCoefmat(x$tableNP4,digits = digits, signif.stars = signif.stars,has.Pvalue = TRUE,na.print = "NA",cs.ind = 1,...)
+        cat("\n")
+      }  
+
+
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+     if( x$margins[1] %in% cont1par && x$margins[2] %in% cont3par ){
+
+
+  cat("\nEQUATION 3")
+  cat("\nLink function for sigma2.2:","log","\n") 
+  cat("Formula: "); print(x$formula[[3]])  
+      cat("\n")
+        cat("Parametric coefficients:\n")
+        printCoefmat(x$tableP3,digits = digits, signif.stars = signif.stars,na.print = "NA",...)
+        cat("\n")
+      
+          if(x$l.sp3!=0){
+          cat("Smooth components' approximate significance:\n")
+          printCoefmat(x$tableNP3,digits = digits, signif.stars = signif.stars,has.Pvalue = TRUE,na.print = "NA",cs.ind = 1,...)
+          cat("\n")
+      }
+  
+ 
+     
+  cat("\nEQUATION 4")
+  cat("\nLink function for nu.2:","log","\n") 
+  cat("Formula: "); print(x$formula[[4]])  
+      cat("\n")
+        cat("Parametric coefficients:\n")
+        printCoefmat(x$tableP4,digits = digits, signif.stars = signif.stars,na.print = "NA",...)
+        cat("\n")
+      
+          if(x$l.sp4!=0){
+          cat("Smooth components' approximate significance:\n")
+          printCoefmat(x$tableNP4,digits = digits, signif.stars = signif.stars,has.Pvalue = TRUE,na.print = "NA",cs.ind = 1,...)
+          cat("\n")
+      }  
+  
+  cat("\nEQUATION 5")
+  cat("\nLink function for theta:",lind,"\n") 
+  cat("Formula: "); print(x$formula[[5]])
+      cat("\n")
+        cat("Parametric coefficients:\n")
+        printCoefmat(x$tableP5,digits = digits, signif.stars = signif.stars,na.print = "NA",...)
+        cat("\n")
+      
+          if(x$l.sp5!=0){
+          cat("Smooth components' approximate significance:\n")
+          printCoefmat(x$tableNP5,digits = digits, signif.stars = signif.stars,has.Pvalue = TRUE,na.print = "NA",cs.ind = 1,...)
+          cat("\n")
+      }  
+
+
+
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
@@ -411,8 +548,8 @@ print.summary.copulaReg <- function(x, digits = max(3, getOption("digits") - 3),
 
   CIrs    <- colMeans(x$CItheta, na.rm = TRUE)
   CIkt    <- colMeans(x$CIkt, na.rm = TRUE)
-  CIsig21 <- colMeans(x$CIsig21, na.rm = TRUE)
-  CIsig22 <- colMeans(x$CIsig22, na.rm = TRUE)
+  if( x$margins[1] %in% c(cont2par,cont3par) ) CIsig21 <- colMeans(x$CIsig21, na.rm = TRUE)
+  if( x$margins[2] %in% c(cont2par,cont3par) ) CIsig22 <- colMeans(x$CIsig22, na.rm = TRUE)
   
   
   if(x$margins[1] %in% cont3par)  CInu1 <- colMeans(x$CInu1, na.rm = TRUE)
@@ -427,6 +564,21 @@ print.summary.copulaReg <- function(x, digits = max(3, getOption("digits") - 3),
                                                                      ct,format(kt.p,digits=nodi),"(",format(CIkt[1],digits=nodi),",",format(CIkt[2],digits=nodi),")",
                                                                      "\nn = ",x$n, "  total edf = ",format(x$t.edf,digits=nodi),"\n\n", sep="")  
 
+
+
+  if( x$margins[1] %in% cont1par && x$margins[2] %in% cont2par ) cat(s2,format(s2.p,digits=nodi),"(",format(CIsig22[1],digits=nodi),",",format(CIsig22[2],digits=nodi),")",
+                                                                     "\ntheta = ",format(as.p,digits=nodi),"(",format(CIrs[1],digits=nodi),",",format(CIrs[2],digits=nodi),")",
+                                                                     ct,format(kt.p,digits=nodi),"(",format(CIkt[1],digits=nodi),",",format(CIkt[2],digits=nodi),")",
+                                                                     "\nn = ",x$n, "  total edf = ",format(x$t.edf,digits=nodi),"\n\n", sep="")  
+
+
+
+if( x$margins[1] %in% cont1par && x$margins[2] %in% cont3par ) cat(s2,format(s2.p,digits=nodi),"(",format(CIsig22[1],digits=nodi),",",format(CIsig22[2],digits=nodi),")",
+                                                                     "\n",n2,format(n2.p,digits=nodi),"(",format(CInu2[1],digits=nodi),",",format(CInu2[2],digits=nodi),")", 
+                                                                     "\ntheta = ",format(as.p,digits=nodi),"(",format(CIrs[1],digits=nodi),",",format(CIrs[2],digits=nodi),")",
+                                                                     ct,format(kt.p,digits=nodi),"(",format(CIkt[1],digits=nodi),",",format(CIkt[2],digits=nodi),")",
+                                                                     "\nn = ",x$n, "  total edf = ",format(x$t.edf,digits=nodi),"\n\n", sep="")                           
+
   if( x$margins[1] %in% cont3par && x$margins[2] %in% cont3par ) cat(s1,format(s1.p,digits=nodi),"(",format(CIsig21[1],digits=nodi),",",format(CIsig21[2],digits=nodi),")",
                                                                      "  ",s2,format(s2.p,digits=nodi),"(",format(CIsig22[1],digits=nodi),",",format(CIsig22[2],digits=nodi),")",
                                                                      "\n",n1,format(n1.p,digits=nodi),"(",format(CInu1[1],digits=nodi),",",format(CInu1[2],digits=nodi),")",
@@ -454,23 +606,3 @@ invisible(x)
                 
 }
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
